from ...types import ExternalAsset


class TableauRevampAsset(ExternalAsset):
    """
    Tableau assets
    """

    COLUMN = "columns"
    DASHBOARD = "dashboards"
    DATASOURCE = "datasources"
    FIELD = "fields"
    PROJECT = "projects"
    SHEET = "sheets"
    TABLE = "tables"
    USAGE = "usage"
    USER = "users"
    WORKBOOK = "workbooks"
