"""
Request timeout in seconds for Looker API
"""

DEFAULT_LOOKER_TIMEOUT_SECOND = 120
KEY_LOOKER_TIMEOUT_SECOND = "CASTOR_LOOKER_TIMEOUT_SECOND"

"""
Number of items per page when requesting Looker API
"""
DEFAULT_LOOKER_PAGE_SIZE = 500
KEY_LOOKER_PAGE_SIZE = "CASTOR_LOOKER_PAGE_SIZE"

"""
Maximum concurrent threads to run when fetching
"""
DEFAULT_LOOKER_THREAD_POOL_SIZE = 20
KEY_LOOKER_THREAD_POOL_SIZE = "CASTOR_LOOKER_THREAD_POOL_SIZE"

# env variables
BASE_URL = "CASTOR_LOOKER_BASE_URL"
CLIENT_ID = "CASTOR_LOOKER_CLIENT_ID"
CLIENT_SECRET = "CASTOR_LOOKER_CLIENT_SECRET"  # noqa: S105
SEARCH_PER_FOLDER = "CASTOR_LOOKER_SEARCH_PER_FOLDER"
LOG_TO_STDOUT = "CASTOR_LOOKER_LOG_TO_STDOUT"
