DEFAULT_API_VERSION = 59.0
DEFAULT_PAGINATION_LIMIT = 100


class Keys:
    """Salesforce's credentials keys"""

    USERNAME = "username"
    PASSWORD = "password"  # noqa: S105
    CLIENT_ID = "client_id"
    CLIENT_SECRET = "client_secret"  # noqa: S105
    SECURITY_TOKEN = "security_token"  # noqa: S105
    BASE_URL = "base_url"
