from ...types import ExternalAsset


class SigmaAsset(ExternalAsset):
    """Sigma assets"""

    DATASETS = "datasets"
    ELEMENTS = "elements"
    FILES = "files"
    LINEAGES = "lineages"
    MEMBERS = "members"
    QUERIES = "queries"
    WORKBOOKS = "workbooks"
